package picocontainer;

import java.util.*;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

public class DefaultPicoContainer implements PicoContainer {

//    private PicoContainer parentContainer;
    private List componentClasses = new ArrayList();
    private Map componentClassToInstanceMap = new HashMap();

//    public DefaultPicoContainer(PicoContainer parentContainer)
//    {
//        this.parentContainer = parentContainer;
//    }

    public DefaultPicoContainer()
    {
    }

    public void registerComponent(Class comp) throws DuplicateComponentRegistrationException
    {
        for (Iterator iterator = componentClasses.iterator(); iterator.hasNext();)
        {
            Class aClass = (Class) iterator.next();
            if (aClass == comp) {
                throw new DuplicateComponentRegistrationException(aClass);
            }
        }
        componentClasses.add(comp);
    }

    public void startup() throws UnsatisfiedDependencyStartupException
    {
        boolean progress = true;
        while (progress == true) {
            progress = false;

            for (Iterator iterator = componentClasses.iterator(); iterator.hasNext();) {
                Class clazz = (Class) iterator.next();
                if (componentClassToInstanceMap.get(clazz) == null)  {
                    // hook'em up
                    try {
                        Constructor[] ctors = clazz.getConstructors();

                        // TODO for now assuming 1 ctor only. fixme
                        Constructor constructor = ctors[0];
                        Class[] params = constructor.getParameterTypes();

                        // For each param, look up the instantiated comp.
                        Object[] args = new Object[params.length];
                        for (int i = 0; i < params.length; i++) {
                            Class param = params[i];
                            args[i] = getComponentForParam(param);
                        }
                        if (hasNullArgs(args) == false) {
                            Object component = null;
                            component = makeComponentInstance(constructor, args);
                            // Put the instantiated comp back in the map
                            componentClassToInstanceMap.put(clazz, component);
                            progress = true;
                        }

                    } catch (IllegalArgumentException e) {
                        e.printStackTrace();  //To change body of catch statement use Options | File Templates.
                    } catch (InvocationTargetException e) {
                        e.getCause().printStackTrace();
                        //e.printStackTrace();  //To change body of catch statement use Options | File Templates.
                    } catch (InstantiationException e) {
                        e.printStackTrace();  //To change body of catch statement use Options | File Templates.
                    } catch (IllegalAccessException e) {
                        e.printStackTrace();  //To change body of catch statement use Options | File Templates.
                    }

                }

            }
        }

        for (Iterator iterator = componentClasses.iterator(); iterator.hasNext();)
        {
            Class component = (Class) iterator.next();
            if (componentClassToInstanceMap.get(component) == null) {
                throw new UnsatisfiedDependencyStartupException(component);
            }

        }
    }

    protected Object makeComponentInstance(Constructor constructor, Object[] args) throws InstantiationException, IllegalAccessException, InvocationTargetException
    {
        return constructor.newInstance(args);
    }

    private Object getComponentForParam(Class param) {
        for (Iterator iterator = componentClassToInstanceMap.entrySet().iterator(); iterator.hasNext();) {
            Map.Entry entry = (Map.Entry) iterator.next();
            Class clazz = (Class) entry.getKey();
            if (param.isAssignableFrom(clazz)) {
                return entry.getValue();
            }
        }
        return null;
    }

    public Object getComponent(Class clazz)
    {
        return componentClassToInstanceMap.get(clazz);
    }

    private boolean hasNullArgs(Object[] args) {
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            if (arg == null) {
                return true;
            }
        }
        return false;
    }

    public boolean hasComponent(Class clazz) {
        return componentClassToInstanceMap.get(clazz) != null;
    }
}
