package picocontainer;

import junit.framework.TestCase;
import picocontainer.DefaultPicoContainer;
import picocontainer.DuplicateComponentRegistrationException;
import picocontainer.PicoContainer;
import picocontainer.PicoRegistrationException;
import picocontainer.PicoStartupException;
import picocontainer.UnsatisfiedDependencyStartupException;
import picocontainer.FredImpl;

public class BasicPicoContainerTestCase extends TestCase {

    public void testBasic() throws PicoStartupException, PicoRegistrationException
    {
        PicoContainer nano = new DefaultPicoContainer();


        nano.registerComponent(FredImpl.class);
        nano.registerComponent(WilmaImpl.class);

        nano.startup();

        assertTrue( "There should have been a Fred in the container", nano.hasComponent( FredImpl.class ) );
        assertTrue( "There should have been a Wilma in the container", nano.hasComponent( WilmaImpl.class ) );

    }

    public void testTooFew() throws PicoStartupException, PicoRegistrationException
    {
        PicoContainer nano = new DefaultPicoContainer();

        nano.registerComponent(FredImpl.class);

        try
        {
            nano.startup();
            fail("should need a wilma");
        }
        catch (UnsatisfiedDependencyStartupException e)
        {
            // expected
            assertTrue(e.getClassThatNeedsDeps() == FredImpl.class);

        }

    }

    public void testDupes() throws PicoStartupException, PicoRegistrationException
    {
        PicoContainer nano = new DefaultPicoContainer();

        nano.registerComponent(WilmaImpl.class);
        try
        {
            nano.registerComponent(WilmaImpl.class);
            fail("Should have barfed with dupe registration");
        }
        catch (DuplicateComponentRegistrationException e)
        {
            // expected
            assertTrue(e.getDuplicateClass() == WilmaImpl.class);
        }


    }


}
